<?php

namespace greenshiftaddon\Blocks;

defined('ABSPATH') or exit;


class DynamicGallery
{

	public function __construct()
	{
		add_action('init', array($this, 'init_handler'));
	}

	public function init_handler()
	{
		register_block_type(
			__DIR__,
			array(
				'render_callback' => array($this, 'render_block'),
				'attributes'      => $this->attributes
			)
		);
	}

	public $attributes = array(
		'id' => array(
			'type'    => 'string',
			'default' => null,
		),
		'inlineCssStyles' => array(
			'type'    => 'string',
			'default' => '',
		),
		'animation' => array(
			'type' => 'object',
			'default' => array(),
		),
		'sourceType'       => array(
			'type'    => 'string',
			'default' => 'latest_item',
		),
		'postId'       => array(
			'type'    => 'number',
			'default' => 0,
		),
		'post_type' => array(
			'type' => 'string',
			'default' => 'post'
		),
		'dynamicField' => array(
			'type' => 'string',
			'default' => ''
		),
		'lightbox' => array(
			'type' => 'boolean',
			'default' => false,
		),
		'field_format' => array(
			'type' => 'string',
			'default' => 'array_ids',
		),
		'isSlider' => array(
			'type' => 'boolean',
			'default' => false,
		),
		'limit' => array(
			'type' => 'number',
			'default' => ''
		),
		'enableThumbnail' => array(
			'type' => 'boolean',
			'default' => false,
		),
	);

	public function render_block($settings = array(), $inner_content = '')
	{
		extract($settings);
		//print_r($settings);
		$blockId = 'gspb_id-' . $id;
		$blockClassName = 'gspb-dynamic-acf-gallery ' . $blockId . ' ' . (!empty($className) ? $className : '') . ' ';

		$out = '<div class="' . $blockClassName . '"' . gspb_AnimationRenderProps($animation) . '>';

		if (empty($dynamicField)) {
			$out = '<div>Please select gallery feild.</div>';
		}

		if (!empty($dynamicField)) {
			if ($sourceType == 'latest_item') {
				global $post;
				$postId = $post->ID;
			} else {
				$postId = (isset($postId) && $postId > 0) ? (int)$postId : 0;
				if ($postId == 0) {
					$args = array(
						'post_type' => $post_type,
						'posts_per_page'  => 1,
						'fields' => 'ids',
						'post_status' => 'publish'
					);
					$latest_cpt = get_posts($args);
					$postId = $latest_cpt[0];
				}
			}

			$result = [];
			$field = $dynamicField;
			if ($field_format == 'array_ids') {
				$galleryIdArray = get_post_meta($postId, $field, true);
				if (!empty($galleryIdArray)) {
					foreach ($galleryIdArray as $image_id) {
						$imgsrc = wp_get_attachment_url($image_id);
						$result[] = $imgsrc;
					}
				}
			} else if ($field_format == 'array_urls') {
				$galleryData = get_post_meta($postId, $field, true);
				if (is_array($galleryData) && !empty($galleryData)) {
					$result = $galleryData;
				}
			} else if ($field_format == 'acf_gallery') {
				if (function_exists('get_field')) {
					$galleryIdArray = get_field($field, $postId);
					if (!empty($galleryIdArray)) {
						foreach ($galleryIdArray as $image_id) {
							if (is_array($image_id)) {
								$imgsrc = $image_id['url'];
							} else if (is_numeric($image_id)) {
								$imgsrc = wp_get_attachment_url($image_id);
							} else {
								$imgsrc = $image_id;
							}
							$result[] = $imgsrc;
						}
					}
				}
			} else if ($field_format == 'post_images') {
				$galleryIdArray = get_attached_media('image', $postId);
				if (!empty($galleryIdArray)) {
					foreach ($galleryIdArray as $image_id) {
						$result[] = $image_id['guid'];
					}
				}
			}
			if ($enableThumbnail) {
				$im = get_the_post_thumbnail_url($postId, 'full');
				if ($im) {
					array_unshift($result, get_the_post_thumbnail_url($postId, 'full'));
				}
			}

			$lightboxClass = $lightbox ? ' gspb_gallery_lightbox' : '';
			$isSliderClass = $isSlider ? ' swiper' : '';
			if (!empty($result)) {
				$out .= '<div class="gspb_gallery_grid' . $lightboxClass . $isSliderClass . '">';
				if ($isSlider) $out .= '<div class="swiper-wrapper">';
				if ($limit && is_array($result)) {
					$result = array_slice($result, 0, $limit);
				}
				foreach ($result as $imagesrc) {
					if ($isSlider) $out .= '<div class="swiper-slide">';
					if ($lightbox) {
						$out .= '<a href="' . $imagesrc . '" class="imagelink">';
						$out .= '<img src="' . $imagesrc . '" alt="" >';
						$out .= '</a>';
					} else {
						$out .= '<img src="' . $imagesrc . '" alt="" >';
					}
					if ($isSlider) {
						$out .= '</div>';
					}
				}
				if ($isSlider) {
					$out .= '</div>';
				}
				$out .= '</div>';
			}
		}


		$out .= '</div>';
		return $out;
	}
}

new DynamicGallery;
