<?php

/**
 * Plugin Name: Greenshift Query and Meta Addon
 * Description: Get any meta value, use better query block
 * Author: Wpsoul
 * Author URI: https://greenshiftwp.com
 * Version: 2.3
 */

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit;
}

// Define Dir URL
define('GREENSHIFTQUERY_DIR_URL', plugin_dir_url(__FILE__));
define('GREENSHIFTQUERY_DIR_PATH', plugin_dir_path(__FILE__));
define('GREENSHIFTQUERY_PLUGIN_VER', '2.3');

function gspb_query_is_parent_active()
{
    $active_plugins = get_option('active_plugins', array());

    if (is_multisite()) {
        $network_active_plugins = get_site_option('active_sitewide_plugins', array());
        $active_plugins         = array_merge($active_plugins, array_keys($network_active_plugins));
    }

    foreach ($active_plugins as $basename) {
        if (
            0 === strpos($basename, 'greenshift-animation-and-page-builder-blocks/')
        ) {
            return true;
        }
    }

    return false;
}

if (gspb_query_is_parent_active()) {
    if (!defined('EDD_CONSTANTS')) {
        require_once GREENSHIFT_DIR_PATH . 'edd/edd_constants.php';
    }

    add_filter('plugins_api', 'greenshiftquery_plugin_info', 20, 3);
    add_filter('site_transient_update_plugins', 'greenshiftquery_push_update');
    add_action('upgrader_process_complete', 'greenshiftquery_after_update', 10, 2);
    add_action('after_plugin_row_' . plugin_basename(__FILE__), 'greenshiftquery_after_plugin_row', 10, 3);

    // Hook: Editor assets.
    add_action('enqueue_block_editor_assets', 'greenShiftQuery_editor_assets');
} else {
    add_action('admin_notices', 'greenshiftquery_admin_notice_warning');
}


//////////////////////////////////////////////////////////////////
// Plugin updater
//////////////////////////////////////////////////////////////////

function greenshiftquery_after_plugin_row($plugin_file, $plugin_data, $status)
{
    $licenses = greenshift_edd_check_all_licenses();
    $is_active = ((!empty($licenses['all_in_one']) && $licenses['all_in_one'] == 'valid') || (!empty($licenses['query_addon']) && $licenses['query_addon'] == 'valid') || (!empty($licenses['all_in_one_design']) && $licenses['all_in_one_design'] == 'valid') || (!empty($licenses['all_in_one_seo']) && $licenses['all_in_one_seo'] == 'valid')) ? true : false;
    if (!$is_active) {
        echo sprintf('<tr class="active"><td colspan="4">%s <a href="%s">%s</a></td></tr>', 'Please enter a license to receive automatic updates', esc_url(admin_url('admin.php?page=' . EDD_GSPB_PLUGIN_LICENSE_PAGE)), 'Enter License.');
    }
}

function greenshiftquery_plugin_info($res, $action, $args)
{

    // do nothing if this is not about getting plugin information
    if ($action !== 'plugin_information') {
        return false;
    }

    // do nothing if it is not our plugin
    if (plugin_basename(__DIR__) !== $args->slug) {
        return $res;
    }

    // trying to get from cache first, to disable cache comment 23,33,34,35,36
    if (false == $remote = get_transient('greenshiftquery_upgrade_pluginslug')) {

        // info.json is the file with the actual information about plug-in on your server
        $remote = wp_remote_get(
            EDD_GSPB_STORE_URL_UPDATE . '/get-info.php?slug=' . plugin_basename(__DIR__) . '&action=info',
            array(
                'timeout' => 15,
                'headers' => array(
                    'Accept' => 'application/json'
                )
            )
        );

        if (!is_wp_error($remote) && isset($remote['response']['code']) && $remote['response']['code'] == 200 && !empty($remote['body'])) {
            set_transient('greenshiftquery_upgrade_pluginslug', $remote, 60000);
        }
    }

    $remote = wp_remote_get(
        EDD_GSPB_STORE_URL_UPDATE . '/get-info.php?slug=' . plugin_basename(__DIR__) . '&action=info',
        array(
            'timeout' => 15,
            'headers' => array(
                'Accept' => 'application/json'
            )
        )
    );

    if ($remote && !is_wp_error($remote)) {

        $remote = json_decode(wp_remote_retrieve_body($remote));

        $res = new stdClass();
        $res->name = $remote->name;
        $res->slug = $remote->slug;
        $res->version = $remote->version;
        $res->tested = $remote->tested;
        $res->requires = $remote->requires;
        $res->author = $remote->author;
        $res->author_profile = $remote->author_homepage;
        $res->download_link = $remote->download_link;
        $res->trunk = $remote->download_link;
        $res->last_updated = $remote->last_updated;

        if (isset($remote->sections)) {
            $res->sections = array(
                'description' => $remote->sections->description, // description tab
                'installation' => $remote->sections->installation, // installation tab
                'changelog' => isset($remote->sections->changelog) ? $remote->sections->changelog : '',
            );
        }
        if (isset($remote->banners)) {
            $res->banners = array(
                'low' => $remote->banners->low,
                'high' => $remote->banners->high,
            );
        }

        return $res;
    }

    return false;
}

function greenshiftquery_push_update($transient)
{

    if (empty($transient->checked)) {
        return $transient;
    }

    // trying to get from cache first, to disable cache comment 11,20,21,22,23
    if (false == $remote = get_transient('greenshiftquery_upgrade_pluginslug')) {
        // info.json is the file with the actual plugin information on your server
        $remote = wp_remote_get(
            EDD_GSPB_STORE_URL_UPDATE . '/get-info.php?slug=' . plugin_basename(__DIR__) . '&action=info',
            array(
                'timeout' => 10,
                'headers' => array(
                    'Accept' => 'application/json'
                )
            )
        );

        if (!is_wp_error($remote) && isset($remote['response']['code']) && $remote['response']['code'] == 200 && !empty($remote['body'])) {
            set_transient('greenshiftquery_upgrade_pluginslug', $remote, 60000);
        }
    }

    if (!is_wp_error($remote) && $remote) {

        $remote = json_decode($remote['body']);

        // your installed plugin version should be on the line below! You can obtain it dynamically of course
        if ($remote && version_compare(GREENSHIFTQUERY_PLUGIN_VER, $remote->version, '<') && version_compare($remote->requires, get_bloginfo('version'), '<')) {
            $res = new stdClass();
            $res->slug = plugin_basename(__DIR__);
            $res->plugin = plugin_basename(__FILE__); // it could be just pluginslug.php if your plugin doesn't have its own directory
            $res->new_version = $remote->version;
            $res->tested = $remote->tested;
            $licenses = greenshift_edd_check_all_licenses();
            $is_active = ((!empty($licenses['all_in_one']) && $licenses['all_in_one'] == 'valid') || (!empty($licenses['query_addon']) && $licenses['query_addon'] == 'valid') || (!empty($licenses['all_in_one_design']) && $licenses['all_in_one_design'] == 'valid') || (!empty($licenses['all_in_one_seo']) && $licenses['all_in_one_seo'] == 'valid')) ? true : false;
            if ($is_active) {
                $res->package = $remote->download_link;
            }
            $transient->response[$res->plugin] = $res;
            //$transient->checked[$res->plugin] = $remote->version;
        }
    }
    return $transient;
}

function greenshiftquery_after_update($upgrader_object, $options)
{
    if ($options['action'] == 'update' && $options['type'] === 'plugin') {
        // just clean the cache when new plugin version is installed
        delete_transient('greenshiftquery_upgrade_pluginslug');
    }
}

function greenshiftquery_admin_notice_warning()
{
?>
    <div class="notice notice-warning">
        <p><?php printf(__('Please, activate %s plugin to use Animation Addon'), '<a href="https://wordpress.org/plugins/greenshift-animation-and-page-builder-blocks" target="_blank">Greenshift</a>'); ?></p>
    </div>
<?php
}

function greenshiftquery_change_action_links($links)
{

    $links = array_merge(array(
        '<a href="https://greenshiftwp.com/changelog" style="color:#93003c" target="_blank">' . __('What\'s New', 'greenshiftquery') . '</a>'
    ), $links);

    return $links;
}
add_action('plugin_action_links_' . plugin_basename(__FILE__), 'greenshiftquery_change_action_links');


/**
 * GreenShift Blocks Category
 */
if (!function_exists('gspb_greenShiftQuery_category')) {
    function gspb_greenShiftQuery_category($categories, $post)
    {
        return array_merge(
            array(
                array(
                    'slug'  => 'greenShiftQuery',
                    'title' => __('GreenShift Query and Meta'),
                ),
            ),
            $categories
        );
    }
}
add_filter('block_categories_all', 'gspb_greenShiftQuery_category', 1, 2);

//////////////////////////////////////////////////////////////////
// Register server side
//////////////////////////////////////////////////////////////////
require_once GREENSHIFTQUERY_DIR_PATH . 'query.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/meta/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/thumbcounter/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/wishlist/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/advanced-listing/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/visibility-block/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/taxonomy/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/login-form/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/dynamic-post-title/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/dynamic-post-image/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/querygrid/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/pagelist/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/dynamicgallery/block.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'blockrender/repeater/block.php';

//REST functions
require_once GREENSHIFTQUERY_DIR_PATH . 'rest.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'hotcounter.php';
require_once GREENSHIFTQUERY_DIR_PATH . 'wishlist.php';

require_once GREENSHIFTQUERY_DIR_PATH . 'querypatterns.php';


//////////////////////////////////////////////////////////////////
// Functions to render conditional scripts
//////////////////////////////////////////////////////////////////

// Hook: Frontend assets.
add_action('init', 'greenShiftQuery_register_init');

if (!function_exists('greenShiftQuery_register_init')) {
    function greenShiftQuery_register_init()
    {
        wp_register_script(
            'gs-thumbscounter',
            GREENSHIFTQUERY_DIR_URL . 'assets/thumbs.js',
            array(),
            '1.6',
            true
        );
        wp_localize_script(
            'gs-thumbscounter',
            'gsthumbvars',
            array(
                'ajax_url' => admin_url('admin-ajax.php', 'relative'),
                'hotnonce' => wp_create_nonce('hotnonce'),
            )
        );

        wp_register_script(
            'gs-wishcounter',
            GREENSHIFTQUERY_DIR_URL . 'assets/wishlist.js',
            array(),
            '1.0',
            true
        );
        wp_localize_script(
            'gs-wishcounter',
            'gswishvars',
            array(
                'ajax_url' => admin_url('admin-ajax.php', 'relative'),
                'wishnonce' => wp_create_nonce('wishnonce'),
            )
        );

        wp_register_script(
            'gsquerytoggler',
            GREENSHIFTQUERY_DIR_URL . 'libs/toggle/toggle.js',
            array(),
            '1.0',
            true
        );

        //filter panel
        wp_register_script(
            'gspbfilterpanel',
            GREENSHIFTQUERY_DIR_URL . 'libs/filterpanel/index.js',
            array(),
            '1.5',
            true
        );
        wp_register_script(
            'gspbajaxpagination',
            GREENSHIFTQUERY_DIR_URL . 'libs/filterpanel/ajaxpagination.js',
            array(),
            '2.0',
            true
        );

        wp_register_style(
            'gsquerytooltip',
            GREENSHIFTQUERY_DIR_URL . 'libs/tooltip.css',
            array(),
            '1.0',
            true
        );

        // login form
        wp_register_script(
            'gspbloginform',
            GREENSHIFTQUERY_DIR_URL . 'libs/loginform/index.js',
            array(),
            '1.1',
            true
        );

        wp_register_script(
            'gsdynamicgallery',
            GREENSHIFTQUERY_DIR_URL . 'libs/dynamicgallery/index.js',
            array(),
            '1.0',
            true
        );
    }
}


//Polyfill for str_contains
if (!function_exists('str_contains')) {
    function str_contains(string $haystack, string $needle)
    {
        return empty($needle) || strpos($haystack, $needle) !== false;
    }
}

add_filter('render_block', 'greenShiftQuery_block_script_assets', 10, 2);
if (!function_exists('greenShiftQuery_block_script_assets')) {
    function greenShiftQuery_block_script_assets($html, $block)
    {
        // phpcs:ignore

        //Main styles for blocks are loaded via Redux. Can be found in src/customJS/editor/store/index.js and src/gspb-library/helpers/reusable_block_css/index.js

        if (!is_admin()) {
            $blockname = $block['blockName'];
            if ($blockname == 'greenshift-blocks/thumbcounter') {
                wp_enqueue_script('gs-thumbscounter');
            } else if ($blockname === 'greenshift-blocks/advanced-listing') {
                if (!empty($block['attrs']['listargs']['togglelink']) && $block['attrs']['listargs']['togglelink'] != 'no') {
                    wp_enqueue_script('gsquerytoggler');
                }
            } else if ($blockname === 'greenshift-blocks/advanced-listing' || $blockname === 'greenshift-blocks/querygrid') {
                if (!empty($block['attrs']['filterpanelenable'])) {
                    wp_enqueue_script('gspbfilterpanel');
                    $scriptvars = array(
                        'filternonce' => wp_create_nonce('filterpanel'),
                        'ajax_url' => admin_url('admin-ajax.php', 'relative'),
                    );
                    wp_localize_script('gspbfilterpanel', 'gspbfiltervars', $scriptvars);
                }
                if (!empty($block['attrs']['enable_pagination']) && ($block['attrs']['enable_pagination'] == '2' || $block['attrs']['enable_pagination'] == '3')) {
                    wp_enqueue_script('gspbajaxpagination');
                    $scriptvars = array(
                        'filternonce' => wp_create_nonce('filterpanel'),
                        'ajax_url' => admin_url('admin-ajax.php', 'relative'),
                    );
                    wp_localize_script('gspbajaxpagination', 'gspbpaginationvars', $scriptvars);
                }
            } else if ($blockname === 'greenshift-blocks/login-form') {
                wp_enqueue_script('gspbloginform');
                $scriptvars = array(
                    'ajax_url' => admin_url('admin-ajax.php', 'relative'),
                );
                wp_localize_script('gspbloginform', 'gspbloginvars', $scriptvars);
            } else if ($blockname == 'greenshift-blocks/dynamicgallery') {
                if (isset($block['attrs']['lightbox']) && $block['attrs']['lightbox'] == true) {
                    wp_enqueue_style('gslightbox');
                    wp_enqueue_script('gslightbox');
                    wp_enqueue_script('gsdynamicgallery');
                }
            }
        }

        return $html;
    }
}

//////////////////////////////////////////////////////////////////
// Enqueue Gutenberg block assets for backend editor.
//////////////////////////////////////////////////////////////////

if (!function_exists('greenShiftQuery_editor_assets')) {
    function greenShiftQuery_editor_assets()
    {
        // phpcs:ignor

        $index_asset_file = include(GREENSHIFTQUERY_DIR_PATH . 'build/index.asset.php');


        // Blocks Assets Scripts
        wp_enqueue_script(
            'greenShiftQuery-block-js', // Handle.
            GREENSHIFTQUERY_DIR_URL . 'build/index.js',
            array('greenShift-editor-js', 'greenShift-library-script', 'wp-block-editor', 'wp-blocks', 'wp-i18n', 'wp-element', 'wp-editor', 'wp-data'),
            $index_asset_file['version'],
            true
        );

        $licenses = greenshift_edd_check_all_licenses();
        $is_active = (((!empty($licenses['all_in_one']) && $licenses['all_in_one'] == 'valid') || (!empty($licenses['query_addon']) && $licenses['query_addon'] == 'valid') || (!empty($licenses['all_in_one_design']) && $licenses['all_in_one_design'] == 'valid') || (!empty($licenses['all_in_one_seo']) && $licenses['all_in_one_seo'] == 'valid'))) ? true : false;

        $check = '';
        if ($is_active) {
            $check = 1;
        }
        $lc = array('can_use_premium_code' => $check);
        wp_localize_script('greenShiftQuery-block-js', 'greenshiftQUERY', $lc);


        // Styles.
        wp_enqueue_style(
            'greenShiftQuery-block-css', // Handle.
            GREENSHIFTQUERY_DIR_URL . 'build/index.css', // Block editor CSS.
            array('greenShift-library-editor', 'wp-edit-blocks'),
            $index_asset_file['version']
        );
    }
}


add_action('init', 'wp_block_custom_meta');
function wp_block_custom_meta()
{
    if (post_type_exists('wp_block')) {
        add_post_type_support('wp_block', 'custom-fields'); // enable custom fields for post type

        //register custom meta fields
        register_post_meta('wp_block', 'gspb_overwrite_archive_by_this_template', array(
            'show_in_rest' => true,
            'single' => true,
            'type' => 'boolean',
            'default' => false,
        ));

        register_post_meta('wp_block', 'gspb_overwrite_filter_by', array(
            'show_in_rest' => true,
            'single' => true,
            'type' => 'string',
            'default' => '',
        ));

        register_post_meta('wp_block', 'gspb_taxonomy_value', array(
            'show_in_rest' => true,
            'single' => true,
            'type' => 'string',
            'default' => '',
        ));

        $tax_args = array(
            'show_in_rest' => array(
                'schema' => array(
                    'type'  => 'array',
                    'items' => array(
                        'type' => 'object',
                        'properties' => array(
                            'id'    => array(
                                'type' => 'string',
                            ),
                            'label' => array(
                                'type'   => 'string',
                            ),
                            'value' => array(
                                'type'   => 'number',
                            ),
                        ),
                    ),
                ),
            ),
            'single' => true,
            'type' => 'array',
        );

        register_post_meta('wp_block', 'gspb_tax_slug', $tax_args);

        register_post_meta('wp_block', 'gspb_tax_slug_exclude', $tax_args);

        register_post_meta('wp_block', 'gspb_roles_list', array(
            'show_in_rest' => array(
                'schema' => array(
                    'type'  => 'array',
                    'items' => array(
                        'type' => 'object',
                        'properties' => array(
                            'id'    => array(
                                'type' => 'string',
                            ),
                            'label' => array(
                                'type'   => 'string',
                            ),
                            'value' => array(
                                'type'   => 'string',
                            ),
                        ),
                    ),
                ),
            ),
            'single' => true,
            'type' => 'array',
        ));

        register_post_meta('wp_block', 'gspb_posttype_archive', array(
            'show_in_rest' => true,
            'single' => true,
            'sanitize_callback' => 'sanitize_text_field',
            'type' => 'string',
        ));

        register_post_meta('wp_block', 'gspb_singular', array(
            'show_in_rest' => true,
            'single' => true,
            'sanitize_callback' => 'sanitize_text_field',
            'type' => 'string',
        ));

        register_post_meta('wp_block', 'gspb_singular_filter_by', array(
            'show_in_rest' => true,
            'single' => true,
            'sanitize_callback' => 'sanitize_text_field',
            'type' => 'string',
            'default' => 'all_items',
        ));

        register_post_meta('wp_block', 'gspb_singular_ids', array(
            'show_in_rest' => array(
                'schema' => array(
                    'type'  => 'array',
                    'items' => array(
                        'type' => 'object',
                        'properties' => array(
                            'id'    => array(
                                'type' => 'string',
                            ),
                            'label' => array(
                                'type'   => 'string',
                            ),
                            'value' => array(
                                'type'   => 'string',
                            ),
                        ),
                    ),
                ),
            ),
            'single' => true,
            'type' => 'array',
        ));
    }
}

add_action('template_redirect', 'gspb_reusable_block_change_template');

function gspb_reusable_block_change_template()
{
    $templates = get_option('gspb_template_replace');
    if (!empty($templates)) {
        foreach ($templates as $key => $template) {
            $type = !empty($template['type']) ? $template['type'] : '';
            $gspb_taxonomy_value = !empty($template['gspb_taxonomy_value']) ? $template['gspb_taxonomy_value'] : '';
            $gspb_tax_slug = !empty($template['gspb_tax_slug']) ? $template['gspb_tax_slug'] : '';
            $gspb_tax_slug_exclude = !empty($template['gspb_tax_slug_exclude']) ? $template['gspb_tax_slug_exclude'] : '';
            $gspb_roles_list = !empty($template['gspb_roles_list']) ? $template['gspb_roles_list'] : '';
            $gspb_posttype_archive = !empty($template['gspb_posttype_archive']) ? $template['gspb_posttype_archive'] : '';
            $gspb_singular = !empty($template['gspb_singular']) ? $template['gspb_singular'] : '';
            $gspb_singular_filter_by = !empty($template['gspb_singular_filter_by']) ? $template['gspb_singular_filter_by'] : '';
            $gspb_singular_ids = !empty($template['gspb_singular_ids']) ? $template['gspb_singular_ids'] : '';

            if ($type) {
                $flag = false;
                if ($type == 'errorpage' && is_404()) {
                    $flag = true;
                } else if ($type == 'singular' && $gspb_singular && is_singular($gspb_singular)) {
                    if ($gspb_singular_filter_by === 'manual_select') {
                        global $post;
                        $posts_include = (!empty($gspb_singular_ids)) ? array_column($gspb_singular_ids, 'id') : '';
                        if (!empty($posts_include) && in_array($post->ID, $posts_include)) $flag = true;
                    } else if ($gspb_singular_filter_by === 'by_taxonomy') {
                        global $post;
                        $post_terms = get_the_terms($post, $gspb_taxonomy_value);
                        if (!$post_terms) continue;

                        $post_terms = array_column($post_terms, 'term_id');

                        $tax_include = (!empty($gspb_tax_slug)) ? array_column($gspb_tax_slug, 'value') : [];
                        $tax_exclude = (!empty($gspb_tax_slug_exclude)) ? array_column($gspb_tax_slug_exclude, 'value') : [];

                        $array_intersect_include = array_values(array_intersect($tax_include, $post_terms));
                        $array_intersect_exclude = array_values(array_intersect($tax_exclude, $post_terms));

                        if (
                            (!empty($tax_exclude) && !empty($array_intersect_exclude)) ||
                            (!empty($tax_include) && empty($array_intersect_include))
                        ) continue;

                        $flag = true;
                    } else {
                        $flag = true;
                    }
                } else if ($type == 'taxonomy' && $gspb_taxonomy_value && (is_tax() || is_category() || is_tag())) {
                    $term = get_queried_object();
                    if ($term->taxonomy != $gspb_taxonomy_value) {
                        continue;
                    }
                    $tax_include = (!empty($gspb_tax_slug)) ? array_column($gspb_tax_slug, 'value') : '';
                    $tax_exclude = (!empty($gspb_tax_slug_exclude)) ? array_column($gspb_tax_slug_exclude, 'value') : '';
                    if (
                        (!empty($tax_include) && !in_array($term->term_id, $tax_include)) ||
                        (!empty($tax_exclude) && in_array($term->term_id, $tax_exclude))
                    ) continue;
                    $flag = true;
                } else if ($type == 'user' && is_author()) {
                    $user_roles = get_queried_object()->roles;
                    $roles_include = (!empty($gspb_roles_list)) ? array_column($gspb_roles_list, 'id') : '';
                    if (!empty($roles_include) && !empty(array_diff($user_roles, $roles_include))) continue;
                    $flag = true;
                } else if ($type == 'posttype' && $gspb_posttype_archive && is_post_type_archive($gspb_posttype_archive)) {
                    $flag = true;
                }
                if ($flag) {
                    $post = get_post($key);
                    if (!$post) {
                        unset($templates[$key]);
                        update_option('gspb_template_replace', $templates);
                        continue;
                    }
                    if ($post->post_status != 'publish') {
                        unset($templates[$key]);
                        update_option('gspb_template_replace', $templates);
                        continue;
                    };
                    $enabled = get_post_meta($key, 'gspb_overwrite_archive_by_this_template', true);
                    if (!$enabled) {
                        unset($templates[$key]);
                        update_option('gspb_template_replace', $templates);
                        continue;
                    }
                    get_header();
                    echo '<main id="site-content">';

                    $content = $post->post_content;
                    $content = do_blocks($content);
                    $content = do_shortcode($content);
                    $content = preg_replace('%<p>&nbsp;\s*</p>%', '', $content);
                    $content = preg_replace('/^(?:<br\s*\/?>\s*)+/', '', $content);
                    echo $content;

                    echo '</main>';
                    get_footer();
                    exit;
                }
            }
        }
    }
}

function gspb_get_post_object_by_id($post_id, $post_type)
{
    if ((int) $post_id > 0) $post = get_post($post_id);
    else {
        $args = array(
            'posts_per_page' => 1,
            'post_status' => 'published',
            'orderby' => 'ID',
            'order' => 'DESC',
            'post_type' => $post_type
        );

        $loop = new WP_Query($args);

        if (empty($loop->posts)) return NULL;

        $post = get_post($loop->posts[0]->ID);
        wp_reset_query();
    }

    return $post;
}
