<?php

// Exit if accessed directly.
if (!defined('ABSPATH')) {
	exit;
}

//////////////////////////////////////////////////////////////////
// REST routes to save and get settings
//////////////////////////////////////////////////////////////////

add_action('rest_api_init', 'gspbquery_register_route');
function gspbquery_register_route()
{
	register_rest_route(
		'greenshift/v1',
		"/metaget/",
		array(
			'methods'  => WP_REST_Server::CREATABLE,
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'callback' => 'gspb_query_metagetapi',
		)
	);
	register_rest_route(
		'greenshift/v1',
		'/update_template_replace/',
		array(
			array(
				'methods'             => 'POST',
				'callback'            => 'gspb_update_template_replace',
				'permission_callback' => function (WP_REST_Request $request) {
					return current_user_can('editor') || current_user_can('administrator');
				}
			),
		)
	);
	register_rest_route(
		'greenshift/v1',
		"/getthumbelement/",
		array(
			'methods'  => WP_REST_Server::CREATABLE,
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'callback' => 'gspb_query_thumbelementapi',
		)
	);
	register_rest_route(
		'greenshift/v1',
		"/getwishlistelement/",
		array(
			'methods'  => WP_REST_Server::CREATABLE,
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'callback' => 'gspb_query_wishlistelementapi',
		)
	);

	register_rest_route('greenshift/v1', '/get-post-types/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_post_types',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			}
		]
	]);

	register_rest_route('greenshift/v1', '/get-post-metas/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_post_metas',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'post_type' => array(
					'type' => 'string',
					'required' => true,
				)
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-taxonomies/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_taxonomies',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'post_type' => array(
					'type' => 'string',
					'required' => true,
				)
			),
		]
	]);
	register_rest_route('greenshift/v1', '/get-all-taxonomies/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_all_taxonomies',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			}
		]
	]);

	register_rest_route('greenshift/v1', '/get-terms/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_terms',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'taxonomy' => array(
					'type' => 'string',
					'required' => true,
				),
				'show_empty' => array(
					'type' => 'boolean',
					'required' => true,
				),
				'order_by' => array(
					'type' => 'string',
					'required' => true,
				),
				'order' => array(
					'type' => 'string',
					'required' => true,
				),
				'include' => array(
					'type' => 'string',
					'required' => false
				),
				'exclude' => array(
					'type' => 'string',
					'required' => false
				),
				'hierarchy' => array(
					'type' => 'boolean',
					'required' => false
				),
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-terms-search/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_taxonomy_terms_search',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'taxonomy' => array(
					'type' => 'string',
					'required' => true,
				),
				'search' => array(
					'type' => 'string',
					'required' => false
				),
				'search-id' => array(
					'type' => 'string',
					'required' => false
				),
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-alpha-html/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_alpha_html',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'taxonomy' => array(
					'type' => 'string',
					'required' => true,
				),
				'show_empty' => array(
					'type' => 'boolean',
					'required' => true,
				),
				'order_by' => array(
					'type' => 'string',
					'required' => true,
				),
				'order' => array(
					'type' => 'string',
					'required' => true,
				),
				'include' => array(
					'type' => 'string',
					'required' => false
				),
				'exclude' => array(
					'type' => 'string',
					'required' => false
				),
				'hierarchy' => array(
					'type' => 'boolean',
					'required' => false
				),
				'show_count' => array(
					'type' => 'boolean',
					'required' => false
				),
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-user-roles/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_user_roles',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'search' => array(
					'type' => 'string',
					'required' => false,
				)
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-post-part/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_post_parts_callback',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'post_id' => array(
					'type' => 'int',
					'required' => false,
				),
				'part' => array(
					'type' => 'string',
					'required' => true,
				),
				'post_type' => array(
					'type' => 'string',
					'required' => true,
				),
				'image_size' => [
					'type' => 'string',
					'required' => false
				]
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-post-by/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_post_by',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'post_id' => array(
					'type' => 'int',
					'required' => false,
				),
				'post_type' => array(
					'type' => 'string',
					'required' => false,
				)
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-post-search/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_post_search',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
			'args' => array(
				'search' => array(
					'type' => 'string',
					'required' => true,
				),
				'post_type' => array(
					'type' => 'string',
					'required' => true,
				)
			),
		]
	]);

	register_rest_route('greenshift/v1', '/get-image-sizes/', [
		[
			'methods' => 'GET',
			'callback' => 'gspb_get_image_sizes',
			'permission_callback' => function (WP_REST_Request $request) {
				return current_user_can('editor') || current_user_can('administrator');
			},
		]
	]);
}

//////////////////////////////////////////////////////////////////
// Get custom value shortcode
//////////////////////////////////////////////////////////////////

if (!function_exists('gspb_query_get_custom_value')) {
	function gspb_query_get_custom_value($atts, $content = null)
	{
		extract(shortcode_atts(array(
			'post_id' => NULL,
			'field' => NULL,
			'subfield' => NULL,
			'subsubfield' => NULL,
			'attrfield' => '',
			'type' => 'custom',
			'show_empty' => '',
			'prefix' => '',
			'postfix' => '',
			'icon' => '',
			'list' => '',
			'showtoggle' => '',
			'imageMapper' => '',
			'post_type' => '',
			'repeaternumber' => '',
			'acfrepeattype' => '',
			'postprocessor' => '',
			'repeaterArray' => '',

		), $atts));
		if (!$field && !$attrfield && !$type) return;
		$field = trim($field);
		$attrfield = trim($attrfield);
		$result = $out = '';
		$field = esc_attr($field);
		$attrfield = esc_attr($attrfield);

		if (!$post_id) {
			global $post;
			if (is_object($post)) {
				$post_id = $post->ID;
			}
		}

		$post_id = (int)$post_id;

		if ($type == 'custom') {
			if (!$field) return;
			$result = GSPB_get_custom_field_value($post_id, $field);
		} else if ($type == 'sitename') {
			$result = get_bloginfo('name');
		} else if ($type == 'sitedescription') {
			$result = get_bloginfo('description');
		} else if ($type == 'currentyear') {
			$result = date_i18n("Y");
		} else if ($type == 'currentmonth') {
			$result = date_i18n("F");
		} else if ($type == 'siteoption') {
			if (!$field) return;
			$field = esc_attr($field);
			$result = GSPB_get_custom_field_value($post_id, $field, ', ', 'option');
		} else if (($type == 'attribute' || $type == 'local') && function_exists('wc_get_product')) {
			if ($post_id) {
				$post_id = trim($post_id);
				$post_id = (int)$post_id;
				$product = wc_get_product($post_id);
				if (!$product) return;
			} else {
				global $product;
				if (!is_object($product)) $product = wc_get_product(get_the_ID());
				if (!$product) return;
			}
			if ($attrfield) $field = $attrfield;
			if (!empty($product)) {
				$woo_attr = $product->get_attribute(esc_html($field));
				if (!is_wp_error($woo_attr)) {
					$result = $woo_attr;
				}
			}
		} else if ($type == 'checkattribute' && function_exists('wc_get_product')) {
			if ($post_id) {
				$post_id = trim($post_id);
				$post_id = (int)$post_id;
				$product = wc_get_product($post_id);
				if (!$product) return;
			} else {
				global $product;
				if (!is_object($product)) $product = wc_get_product(get_the_ID());
				if (!$product) return;
			}
			if ($attrfield) $field = $attrfield;
			if (!empty($product)) {
				$woo_attr = $product->get_attribute(esc_html($field));
				if (!is_wp_error($woo_attr)) {
					$result = $woo_attr;
				}
			}
			if (!empty($result)) {
				$content = do_shortcode($content);
				$content = preg_replace('%<p>&nbsp;\s*</p>%', '', $content);
				$content = preg_replace('/^(?:<br\s*\/?>\s*)+/', '', $content);
				return $content;
			}
			return false;
		} else if ($type == 'vendor') {
			$vendor_id = get_query_var('author');
			if (!empty($vendor_id)) {
				$result = get_user_meta($vendor_id, $field, true);
			}
		} else if ($type == 'taxonomy') {
			$terms = get_the_terms($post_id, esc_html($field));
			if ($terms && !is_wp_error($terms)) {
				$term_slugs_arr = array();
				foreach ($terms as $term) {
					$term_slugs_arr[] = '' . $term->name . '';
				}
				$terms_slug_str = join(", ", $term_slugs_arr);
				$result = $terms_slug_str;
			}
		} else if ($type == 'taxonomylink') {
			$term_list = get_the_term_list($post_id, esc_html($field), '', ', ', '');
			if (!is_wp_error($term_list)) {
				$result = $term_list;
			}
		} else if ($type == 'author' || $type == 'author_meta') {
			$author_id = get_post_field('post_author', $post_id);
			if (!empty($author_id)) {
				$result = get_user_meta($author_id, $field, true);
			}
		} else if ($type == 'author_name') {
			$author_id = get_post_field('post_author', $post_id);
			if (!empty($author_id)) {
				$result = get_the_author_meta('display_name', $author_id);
			}
		} else if ($type == 'author_name_link') {
			$author_id = get_post_field('post_author', $post_id);
			if (!empty($author_id)) {
				$link = get_author_posts_url($author_id);
				$name = get_the_author_meta('display_name', $author_id);
				$result = '<a href=' . $link . '>' . $name . '</a>';
			}
		} else if ($type == 'author_description') {
			$author_id = get_post_field('post_author', $post_id);
			if (!empty($author_id)) {
				$result = get_the_author_meta('user_description', $author_id);
			}
		} else if ($type == 'post_date') {
			$result = get_the_date('', $post_id);
		} else if ($type == 'comment_count') {
			$result = get_post_field('comment_count', $post_id);
		} else if ($type == 'post_modified') {
			$result = get_the_modified_date('', $post_id);
		} else if ($type == 'excerpt') {
			$result = get_the_excerpt($post_id);
		} else if ($type == 'date') {
			if ($field == 'year') {
				return date_i18n("Y");
			} else if ($field == 'month') {
				return date_i18n("F");
			}
		} else if ($type == 'attributelink') {
			if ($attrfield) $field = $attrfield;
			if (function_exists('wc_get_product_terms')) {
				$attribute_values = wc_get_product_terms($post_id, $field, array('fields' => 'all'));
				$values = array();
				foreach ($attribute_values as $attribute_value) {
					$value_name = esc_html($attribute_value->name);
					$values[] = '<a href="' . esc_url(get_term_link($attribute_value->term_id, $field)) . '" rel="tag">' . $value_name . '</a>';
				}
				$result = implode(', ', $values);
			}
		} else if ($type == 'checkmeta') {
			$result = get_post_meta($post_id, $field, true);
			if (!empty($result)) {
				$content = do_shortcode($content);
				$content = preg_replace('%<p>&nbsp;\s*</p>%', '', $content);
				$content = preg_replace('/^(?:<br\s*\/?>\s*)+/', '', $content);
				return $content;
			}
			return false;
		} else if ($type == 'acfmulti' && function_exists('get_field')) {
			$result = get_field($field, $post_id);
			if (!empty($result) && is_array($result)) {
				$result = implode(', ', $result);
			}
		} else if ($type == 'acf_file' && function_exists('get_field')) {
			$result = get_field($field, $post_id);
			if (!empty($result) && is_array($result)) {
				$url = $result['url'];
				$title = $result['filename'];
				$result = '<a href="' . $url . '">' . $title . '</a>';
			}
		} else if ($type == 'acfimage' && function_exists('get_field')) {
			$result = get_field($field, $post_id);
			if (!empty($result) && is_array($result)) {
				$id = $result['id'];
			} else {
				$id = $result;
			}
			if (is_numeric($id)) {
				$result = wp_get_attachment_image($id, 'full');
			} else {
				$result = '<img src=' . $id . ' />';
			}
			return $result;
		} else if ($type == 'acfrepeater' && function_exists('get_field')) {
			$result = get_field($field, $post_id);
			if (!empty($result) && !empty($subfield) && is_array($result)) {
				$rownumber = $repeaternumber ? intval($repeaternumber - 1) : 0;
				$result = $result[$rownumber][$subfield];
				if (!empty($subsubfield)) {
					$result = $result[$rownumber][$subfield][$subsubfield];
				}
				if (!empty($result) && $acfrepeattype == 'multi') {
					$result = implode(', ', $result);
				} else if (!empty($result) && $acfrepeattype == 'image') {
					if (!empty($result) && is_array($result)) {
						$id = $result['id'];
					} else {
						$id = $result;
					}
					if (is_numeric($id)) {
						$result = wp_get_attachment_image($id, 'full');
					} else {
						$result = '<img src=' . $id . ' />';
					}
				} else if (is_array($result)) {
					$result = $result[0];
				}
			}
			return $result;
		} else if ($type == 'acfrepeatertable' && function_exists('get_field')) {
			$getrepeatable = get_field($field, $post_id);
			//print_r($getrepeatable);
			if (!empty($getrepeatable) && is_array($getrepeatable)) {
				$firstrow = $getrepeatable[0];
				$titlearray = array();
				$rowcount = 0;
				while (have_rows($field, $post_id)) : the_row();
					$rowcount++;
					if ($rowcount == 1) {
						foreach ($firstrow as $rowkey => $rowvalue) {
							$current = get_sub_field_object($rowkey);
							$titlearray[] = $current['label'];
						}
					}
				endwhile;
				$result = '<table>';
				$result .= '<tr>';
				foreach ($titlearray as $title) {
					$result .= '<th>' . $title . '</th>';
				}
				$result .= '</tr>';
				foreach ($getrepeatable as $item => $value) {
					$result .= '<tr>';
					foreach ($value as $field) {
						$result .= '<td>';
						if (is_array($field)) {
							if (!empty($field['id'])) {
								$result .= wp_get_attachment_image($field['id'], 'full');
							} else {
								$result .= implode(', ', $field);
							}
						} else {
							$result .= $field;
						}
						$result .= '</td>';
					}
					$result .= '</tr>';
				}
				$result .= '</table>';
			}
			return $result;
		} else if ($type == 'repeater') {
			if (!empty($repeaterArray) && !empty($field)) {
				$result = !empty($repeaterArray[$field]) ? $repeaterArray[$field] : '';
				$result = GSPB_field_array_to_value($result, ', ');
			}
		} else {
			if (function_exists('GSPB_get_custom_field_value')) {
				$result = GSPB_get_custom_field_value($post_id, $field);
			} else {
				$result = get_post_meta($post_id, $field, true);
			}
		}
		if ($type != 'acfmulti' && $type != 'acfimage' && $type != 'acfrepeater' && $type != 'acfrepeatertable') {
			if (!empty($subfield) && !empty($subsubfield) && is_array($result)) {
				$result = $result[$subfield][$subsubfield];
			} else if (!empty($subfield) && is_array($result)) {
				$result = $result[$subfield];
			} else if (is_array($result) && !empty($result[0])) {
				$result = $result[0];
			}
		}
		if ($result) {
			if ($icon && $postprocessor != 'commatolist') {
				$out .= '<span class="gspb_meta_prefix_icon">' . greenshift_render_icon_module($icon) . '</span>';
			}
			if ($prefix) {
				$out .= '<span class="gspb_meta_prefix">' . esc_attr($prefix) . '</span> ';
			}
			if ($showtoggle) {
				$out .= '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" width="30" height="30" fill="green"><path d="M504 256c0 136.967-111.033 248-248 248S8 392.967 8 256 119.033 8 256 8s248 111.033 248 248zM227.314 387.314l184-184c6.248-6.248 6.248-16.379 0-22.627l-22.627-22.627c-6.248-6.249-16.379-6.249-22.628 0L216 308.118l-70.059-70.059c-6.248-6.248-16.379-6.248-22.628 0l-22.627 22.627c-6.248 6.248-6.248 16.379 0 22.627l104 104c6.249 6.249 16.379 6.249 22.628.001z"/></svg>';
			} else {
				$out .= '<div class="gspb_meta_value">';
				$key = '';
				if (!empty($imageMapper)) {
					$key = array_search($result, $imageMapper);
					if ($key) {
						$out .= wp_get_attachment_image((int)$key, 'full');
					}
				}
				if (!$key) {
					if ($postprocessor == 'textformat') {
						$out .= wpautop(wptexturize($result));
					} else if ($postprocessor == 'ymd') {
						$date = DateTime::createFromFormat('Ymd', $result);
						$out .= $date ? wp_date(get_option('date_format'), $date->format('U')) : $result;
					} else if ($postprocessor == 'ytmd') {
						$date = DateTime::createFromFormat('Y-m-d', $result);
						$out .= $date ? wp_date(get_option('date_format'), $date->format('U')) : $result;
					} else if ($postprocessor == 'mailto') {
						$out .= '<a href="mailto:' . $result . '">' . $result . '</a>';
					}else if ($postprocessor == 'tel') {
						$out .= '<a href="tel:' . $result . '">' . $result . '</a>';
					} else if ($postprocessor == 'postlink') {
						$out .= '<a href="' . get_permalink($post_id) . '">' . $result . '</a>';
					} else if ($postprocessor == 'extlink') {
						$out .= '<a href="' . esc_url($result) . '">' . $result . '</a>';
					} else if ($postprocessor == 'commatolist') {
						if (strpos($result, ', ') !== 'false') {
							$result = explode(', ', $result);
						} else if (strpos($result, ',') !== 'false') {
							$result = explode(',', $result);
						}
						if ($icon) {
							$icon = '<span class="gspb_meta_prefix_icon">' . greenshift_render_icon_module($icon) . '</span>';
						} else {
							$icon = '';
						}
						$out .= '<ul><li>' . $icon . implode('</li><li>' . $icon, $result) . '</li></ul>';
					} else {
						if (is_array($result) && count($result) == count($result, COUNT_RECURSIVE)) {
							$result = implode(', ', $result);
						}
						$out .= $result;
					}
				}
				$out .= '</div>';
			}

			if ($postfix) {
				$out .= '<span class="gspb_meta_postfix">' . esc_attr($postfix) . '</span> ';
			}
		} else {
			if ($show_empty) {
				if ($icon) {
					$out .= '<span class="gspb_meta_prefix_icon">' . greenshift_render_icon_module($icon) . '</span>';
				}
				if ($prefix) {
					$out .= '<span class="gspb_meta_prefix">' . esc_attr($prefix) . '</span> ';
				}
				if ($showtoggle) {
					$out .= '<svg xmlns="http://www.w3.org/2000/svg" width="30" height="30" viewBox="0 0 512 512" fill="red"><path d="M256 8C119 8 8 119 8 256s111 248 248 248 248-111 248-248S393 8 256 8zm0 448c-110.5 0-200-89.5-200-200S145.5 56 256 56s200 89.5 200 200-89.5 200-200 200zm101.8-262.2L295.6 256l62.2 62.2c4.7 4.7 4.7 12.3 0 17l-22.6 22.6c-4.7 4.7-12.3 4.7-17 0L256 295.6l-62.2 62.2c-4.7 4.7-12.3 4.7-17 0l-22.6-22.6c-4.7-4.7-4.7-12.3 0-17l62.2-62.2-62.2-62.2c-4.7-4.7-4.7-12.3 0-17l22.6-22.6c4.7-4.7 12.3-4.7 17 0l62.2 62.2 62.2-62.2c4.7-4.7 12.3-4.7 17 0l22.6 22.6c4.7 4.7 4.7 12.3 0 17z"/></svg>';
				} else {
					$out .= '-';
				}
			}
		}
		return $out;
	}
}

function gspb_get_image_sizes()
{
	$res = [];
	foreach (wp_get_registered_image_subsizes() as $key => $val) {
		$res[] = ['label' => $key, 'value' => $key];
	}
	$res[] = ['label' => 'full', 'value' => 'full'];
	return $res;
}

function gspb_get_post_search(WP_REST_Request $request)
{
	$search = sanitize_text_field($request->get_param('search'));
	$post_type = sanitize_text_field($request->get_param('post_type'));

	$args = array(
		'post_type' => $post_type,
		's' => $search
	);

	$query = new WP_Query($args);

	$res = [];
	foreach ($query->posts as $_post) {
		$res[] = ['id' => $_post->ID, 'title' => ['rendered' => $_post->post_title]];
	}
	return $res;
}

function gspb_update_template_replace(WP_REST_Request $request)
{
	try {
		$type = sanitize_text_field($request->get_param('type'));
		$id = sanitize_text_field($request->get_param('id'));
		$gspb_taxonomy_value = sanitize_text_field($request->get_param('gspb_taxonomy_value'));
		$gspb_tax_slug = $request->get_param('gspb_tax_slug');
		$gspb_tax_slug_exclude = $request->get_param('gspb_tax_slug_exclude');
		$gspb_roles_list = $request->get_param('gspb_roles_list');
		$gspb_posttype_archive = sanitize_text_field($request->get_param('gspb_posttype_archive'));
		$gspb_singular = sanitize_text_field($request->get_param('gspb_singular'));
		$gspb_singular_filter_by = sanitize_text_field($request->get_param('gspb_singular_filter_by'));
		$gspb_singular_ids = $request->get_param('gspb_singular_ids');

		$defaults = get_option('gspb_template_replace');
		if ($defaults === false) {
			$defaults = [];
		}
		$defaults[$id]['type'] = $type;
		$defaults[$id]['gspb_taxonomy_value'] = $gspb_taxonomy_value;
		$defaults[$id]['gspb_tax_slug'] = $gspb_tax_slug;
		$defaults[$id]['gspb_tax_slug_exclude'] = $gspb_tax_slug_exclude;
		$defaults[$id]['gspb_roles_list'] = $gspb_roles_list;
		$defaults[$id]['gspb_posttype_archive'] = $gspb_posttype_archive;
		$defaults[$id]['gspb_singular'] = $gspb_singular;
		$defaults[$id]['gspb_singular_filter_by'] = $gspb_singular_filter_by;
		$defaults[$id]['gspb_singular_ids'] = $gspb_singular_ids;

		update_option('gspb_template_replace', $defaults);

		return json_encode(array(
			'success' => true,
			'message' => 'Template updated!',
		));
	} catch (Exception $e) {
		return json_encode(array(
			'success' => false,
			'message' => $e->getMessage(),
		));
	}
}

function gspb_get_post_by(WP_REST_Request $request)
{
	$postId = intval($request->get_param('post_id'));
	$post_type = sanitize_text_field($request->get_param('post_type'));
	$_post = gspb_get_post_object_by_id($postId, $post_type);
	return [['id' => $_post->ID, 'title' => ['rendered' => $_post->post_title]]];
}

function gspb_get_post_parts_callback(WP_REST_Request $request)
{
	$postId = intval($request->get_param('post_id'));
	$part = sanitize_text_field($request->get_param('part'));
	$post_type = sanitize_text_field($request->get_param('post_type'));
	$image_size = sanitize_text_field($request->get_param('image_size'));
	$additional = sanitize_text_field($request->get_param('additional'));
	$additional2 = sanitize_text_field($request->get_param('additional2'));
	$field = sanitize_text_field($request->get_param('field'));

	if ($postId == 0 && $post_type && $part != 'site_data') {
		if ($part == 'taxonomy_field_get_text' || $part == 'taxonomy_field_get_image') {
			$taxonomies = get_terms(array('taxonomy' => $post_type, 'hide_empty' => false, 'number' => 1));
			$postId = $taxonomies[0]->term_id;
		} else {
			$args = array(
				'post_type' => $post_type,
				'posts_per_page'  => 1,
				'post_status' => 'publish'
			);
			$latest_cpt = get_posts($args);
			$postId = $latest_cpt[0]->ID;
		}
	}

	$result = '';
	switch ($part) {
		case 'title':
			$result = !empty(get_the_title($postId)) ? get_the_title($postId) : __('Post don\'t has a title.', 'greenshiftquery');
			break;
		case 'featured_image':
			$result = get_the_post_thumbnail($postId, $image_size);
			$result = !empty($result) ? $result : '<svg width="500" height="500" class="gspb_svg_placeholder" viewBox="0 0 500 500" fill="none" xmlns="http://www.w3.org/2000/svg">
			<rect width="500" height="500" fill="transparent"/>
			<g clip-path="url(#clip0_1521_865)">
			<path d="M196 262.573L226.452 231.96L285.38 291" stroke="black"/>
			<path d="M271.8 230.867C271.8 236.63 267.142 241.3 261.4 241.3C255.658 241.3 251 236.63 251 230.867C251 225.103 255.658 220.433 261.4 220.433C267.142 220.433 271.8 225.103 271.8 230.867Z" stroke="black"/>
			<path d="M259.22 264.76L282.11 241.8L305 265.853" stroke="black"/>
			</g>
			<rect x="196.5" y="209.5" width="108" height="81" stroke="black"/>
			<defs>
			<clipPath id="clip0_1521_865">
			<rect x="196" y="209" width="109" height="82" fill="white"/>
			</clipPath>
			</defs>
			</svg>';
			break;
		case 'meta_field_image':
			$result = GSPB_get_custom_field_value($postId, $field, 'no');
			if (is_numeric($result)) {
				$result = wp_get_attachment_image($result, $image_size);
				return json_encode($result);
			}
			$result = !empty($result) ? '<img src="'.esc_url($result).'" />' : '<svg width="500" height="500" class="gspb_svg_placeholder" viewBox="0 0 500 500" fill="none" xmlns="http://www.w3.org/2000/svg">
			<rect width="500" height="500" fill="transparent"/>
			<g clip-path="url(#clip0_1521_865)">
			<path d="M196 262.573L226.452 231.96L285.38 291" stroke="black"/>
			<path d="M271.8 230.867C271.8 236.63 267.142 241.3 261.4 241.3C255.658 241.3 251 236.63 251 230.867C251 225.103 255.658 220.433 261.4 220.433C267.142 220.433 271.8 225.103 271.8 230.867Z" stroke="black"/>
			<path d="M259.22 264.76L282.11 241.8L305 265.853" stroke="black"/>
			</g>
			<rect x="196.5" y="209.5" width="108" height="81" stroke="black"/>
			<defs>
			<clipPath id="clip0_1521_865">
			<rect x="196" y="209" width="109" height="82" fill="white"/>
			</clipPath>
			</defs>
			</svg>';
			break;
		case 'featured_image_get':
			$result = get_the_post_thumbnail_url($postId, $image_size);
			$result = !empty($result) ? $result : '';
			break;
		case 'meta_field_gallery':
			if ($additional == 'array_ids') {
				$result = [];
				$galleryIdArray = get_post_meta($postId, $field, true);
				if (!empty($galleryIdArray)) {
					foreach ($galleryIdArray as $image_id) {
						$imgsrc = wp_get_attachment_url($image_id);
						$result[] = $imgsrc;
					}
				}
			} else if ($additional == 'array_urls') {
				$galleryData = get_post_meta($postId, $field, true);
				if (is_array($galleryData) && !empty($galleryData)) {
					$result = $galleryData;
				}
			} else if ($additional == 'acf_gallery') {
				if (function_exists('get_field')) {
					$result = [];
					$galleryIdArray = get_field($field, $postId);
					if (!empty($galleryIdArray)) {
						foreach ($galleryIdArray as $image_id) {
							if (is_array($image_id)) {
								$imgsrc = $image_id['url'];
							} else if (is_numeric($image_id)) {
								$imgsrc = wp_get_attachment_url($image_id);
							} else {
								$imgsrc = $image_id;
							}
							$result[] = $imgsrc;
						}
					}
				}
			} else if ($additional == 'post_images') {
				$result = [];
				$galleryIdArray = get_attached_media('image', $postId);
				if (!empty($galleryIdArray)) {
					foreach ($galleryIdArray as $image_id) {
						$result[] = $image_id['guid'];
					}
				}
			}
			if ($additional2 === 'true') {
				$im = get_the_post_thumbnail_url($postId, 'full');
				if ($im) {
					array_unshift($result, get_the_post_thumbnail_url($postId, 'full'));
				}
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'meta_repeater':
			$result = [];
			if ($additional == 'acf' && function_exists('get_field')) {
				$getrepeatable = get_field($field, $postId);
			} else if ($additional == 'relationpostobj' || $additional == 'relationpostids') {
				if (function_exists('get_field')) {
					$getrepeatable = get_field($field, $postId);
				} else {
					$getrepeatable = get_post_meta($postId, $field, true);
				}
				if ($additional == 'relationpostids') {
					if (!empty($getrepeatable) && !is_array($getrepeatable)) {
						$ids = wp_parse_id_list($getrepeatable);
					} else {
						$ids = $getrepeatable;
					}
					if(!empty($ids)){
						$args = array(
							'post__in' => $ids,
							'numberposts' => '-1',
							'orderby' => 'post__in',
							'ignore_sticky_posts' => 1,
							'post_type' => 'any'
						);
						$getrepeatable = get_posts($args);
					}
				}
				if (!empty($getrepeatable)) {
					if (!is_array($getrepeatable)) {
						$getrepeatable = [$getrepeatable];
					}
					$posts = [];
					foreach ($getrepeatable as $key => $value) {
						if (is_object($value) && !empty($value->ID)) {
							$posts[$key] = (array) $value;
							$posts[$key]['thumbnail_url'] = get_the_post_thumbnail_url($value->ID, 'full');
							$posts[$key]['author'] = get_the_author_meta('display_name', $value->post_author);
							$posts[$key]['date'] = get_the_date('', $value->ID);
							$posts[$key]['modified_date'] = get_the_modified_date('', $value->ID);

							$remove_keys = ['post_author', 'post_date', 'post_date_gmt', 'post_content', 'post_status', 'comment_status', 'ping_status', 'post_password', 'to_ping', 'pinged', 'post_modified', 'post_modified_gmt', 'post_content_filtered', 'post_parent', 'menu_order', 'post_type', 'post_mime_type', 'filter'];
							foreach ($remove_keys as $keyname) {
								unset($posts[$key][$keyname]);
							}
							$custom_fields = get_post_meta($value->ID);
							foreach ($custom_fields as $fieldindex => $fieldvalue) {
								if (is_serialized($fieldvalue[0])) {
									$fieldvalue[0] = maybe_unserialize($fieldvalue[0]);
								}
								if (!empty($fieldvalue[0])) {
									$posts[$key][$fieldindex] = GSPB_field_array_to_value($fieldvalue[0], ', ');
								}
							}
						}
					}
					$getrepeatable = $posts;
				}
			} else {
				$getrepeatable = GSPB_get_custom_field_value($postId, $field, 'no');
			}
			if (!empty($getrepeatable) && is_array($getrepeatable)) {
				$result = $getrepeatable;
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'site_data':
			if ($field == 'name' || $field == 'description') {
				$result = get_bloginfo($field);
			} else if ($field == 'year') {
				$result = date_i18n("Y");
			} else if ($field == 'month') {
				$result = date_i18n("F");
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'post_data':
			$resultObj = get_post($postId);
			if (is_object($resultObj)) $result = $resultObj->$field;
			if ($result) {
				if ($field == 'post_date') {
					$result = get_the_date('', $postId);
				} else if ($field == 'post_modified') {
					$result = get_the_modified_date('', $postId);
				}
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'author_data':
			$resultObj = get_post($postId);
			if (is_object($resultObj)) $result = $resultObj->post_author;
			if ($result) {
				$result = get_the_author_meta($field, $result);
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'author_meta':
			$resultObj = get_post($postId);
			if (is_object($resultObj)) $resultId = $resultObj->post_author;
			if ($resultId) {
				$result = get_user_meta($resultId, $field, true);
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'meta_field_get_image':
			$result = GSPB_get_custom_field_value($postId, $field, 'no');
			if (is_numeric($result)) $result = wp_get_attachment_url($result);
			$result = !empty($result) ? $result : '';
			break;
		case 'taxonomy_field_get_image':
			$result = get_term_meta($postId, $field, true);
			if (is_array($result)) $result = $result[0];
			if (is_numeric($result)) $result = wp_get_attachment_url($result);
			$result = !empty($result) ? $result : '';
			break;
		case 'taxonomy_value':
			$terms = get_the_terms($postId, $field);
			$divider = ', ';
			if ($additional) $divider = '<span class="gspb_tax_spacer">' . $additional . '</span>';
			if ($terms && !is_wp_error($terms)) {
				$term_slugs_arr = array();
				foreach ($terms as $term) {
					$term_slugs_arr[] = '' . $term->name . '';
				}
				$terms_slug_str = join($divider, $term_slugs_arr);
				$result = $terms_slug_str;
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'taxonomy_link':
			$divider = ', ';
			if ($additional) $divider = '<span class="gspb_tax_spacer">' . $additional . '</span>';
			$term_list = get_the_term_list($postId, $field, '', $divider, '');
			if (!is_wp_error($term_list)) {
				$result = $term_list;
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'meta_field_get_text':
			$result = GSPB_get_custom_field_value($postId, $field);
			$result = !empty($result) ? $result : '';
			break;
		case 'meta_field_get_map':
			$result = GSPB_get_custom_field_value($postId, $field, 'flat');
			if (is_array($result)) {
				$map = [];
				if (isset($result['location']) && isset($result['location']['lat']) && isset($result['location']['lng'])) {
					$map['lat'] = $result['location']['lat'];
					$map['lng'] = $result['location']['lng'];
					$map['title'] = !empty($result['title']) ? $result['title'] : '';
					$map['description'] = !empty($result['description']) ? $result['description'] : '';
				} else if (isset($result['lat']) && isset($result['lng'])) {
					$map['lat'] = $result['lat'];
					$map['lng'] = $result['lng'];
				} else if (isset($result['latitude']) && isset($result['longitude'])) {
					$map['lat'] = $result['latitude'];
					$map['lng'] = $result['longitude'];
				}
				$result = $map;
			}
			$result = !empty($result) ? $result : '';
			break;
		case 'taxonomy_field_get_text':
			if ($field == 'name') {
				$taxobj = get_term($postId);
				if (!is_wp_error($taxobj)) {
					$result = $taxobj->name;
				}
			} else if ($field == 'description') {
				$taxobj = get_term($postId);
				if (!is_wp_error($taxobj)) {
					$result = $taxobj->description;
				}
			} else {
				$result = get_term_meta($postId, $field, true);
			}
			if (is_array($result)) $result = $result[0];
			$result = !empty($result) ? $result : '';
			break;
		default:
			break;
	}

	return json_encode($result);
}

function gspb_get_user_roles(WP_REST_Request $request)
{
	$search = sanitize_text_field($request->get_param('search'));

	global $wp_roles;

	$res = [];

	foreach ($wp_roles->roles as $key => $role) {
		if (empty($search) || strpos(strtolower($role['name']), strtolower($search)) !== false) {
			$res[] = ['label' => $role['name'], 'id' => $key, 'value' => $key];
		}
	}

	return json_encode($res);
}

function gspb_get_alpha_html(WP_REST_Request $request)
{

	$taxonomy = sanitize_text_field($request->get_param('taxonomy'));
	$show_empty = boolval(sanitize_text_field($request->get_param('show_empty')));
	$order_by = sanitize_text_field($request->get_param('order_by'));
	$order = sanitize_text_field($request->get_param('order'));
	$include = sanitize_text_field($request->get_param('include'));
	$exclude = sanitize_text_field($request->get_param('exclude'));
	$hierarchy = boolval(sanitize_text_field($request->get_param('hierarchy')));
	$show_count = sanitize_text_field($request->get_param('show_count'));

	$terms = \Greenshift\Blocks\ProductTaxonomy::get_terms($taxonomy, $show_empty, $order_by, $order, $include, $exclude, $hierarchy);
	$alpha_html = \Greenshift\Blocks\ProductTaxonomy::alphabetical_view($terms, $taxonomy, $show_count, '');
	$json = json_encode($alpha_html);

	return $json;
}

function gspb_get_taxonomy_terms_search(WP_REST_Request $request)
{
	$taxonomy = sanitize_text_field($request->get_param('taxonomy'));
	$search = sanitize_text_field($request->get_param('search'));
	$search_id = sanitize_text_field($request->get_param('search-id'));

	global $wpdb;
	if (empty($search_id)) {
		$query = [
			"select" => "SELECT SQL_CALC_FOUND_ROWS a.term_id AS id, b.name as name, b.slug AS slug
                        FROM {$wpdb->term_taxonomy} AS a
                        INNER JOIN {$wpdb->terms} AS b ON b.term_id = a.term_id",
			"where"  => "WHERE a.taxonomy = '{$taxonomy}'",
			"like"   => "AND (b.slug LIKE '%s' OR b.name LIKE '%s' )",
			"offset" => "LIMIT %d, %d"
		];

		$search_term = '%' . $wpdb->esc_like($search) . '%';
		$offset = 0;
		$search_limit = 100;

		$final_query = $wpdb->prepare(implode(' ', $query), $search_term, $search_term, $offset, $search_limit);
	} else {
		$search_id = rtrim($search_id, ',');
		$query = [
			"select" => "SELECT SQL_CALC_FOUND_ROWS a.term_id AS id, b.name as name, b.slug AS slug
                        FROM {$wpdb->term_taxonomy} AS a
                        INNER JOIN {$wpdb->terms} AS b ON b.term_id = a.term_id",
			"where"  => "WHERE a.taxonomy = '{$taxonomy}'",
			"like"   => "AND a.term_id IN({$search_id})",
			"offset" => "LIMIT %d, %d"
		];

		$offset = 0;
		$search_limit = 100;

		$final_query = $wpdb->prepare(implode(' ', $query), $offset, $search_limit);
	}
	// Return saved values

	$results = $wpdb->get_results($final_query);

	$total_results = $wpdb->get_row("SELECT FOUND_ROWS() as total_rows;");
	$response_data = [];

	if ($results) {
		foreach ($results as $result) {
			$response_data[] = [
				'slug'    	=> esc_html($result->slug),
				'name'  	=> esc_html($result->name),
				'id' 	=> (int)$result->id
			];
		}
	}

	return json_encode($response_data);
}

function gspb_get_terms(WP_REST_Request $request)
{
	$taxonomy = sanitize_text_field($request->get_param('taxonomy'));
	$show_empty = boolval(sanitize_text_field($request->get_param('show_empty')));
	$order_by = sanitize_text_field($request->get_param('order_by'));
	$order = sanitize_text_field($request->get_param('order'));
	$include = sanitize_text_field($request->get_param('include'));
	$exclude = sanitize_text_field($request->get_param('exclude'));
	$hierarchy = boolval(sanitize_text_field($request->get_param('hierarchy')));
	$image_meta = sanitize_text_field($request->get_param('image_meta'));

	$terms = \Greenshift\Blocks\ProductTaxonomy::get_terms($taxonomy, $show_empty, $order_by, $order, $include, $exclude, $hierarchy, false, false, $image_meta);
	$json = json_encode($terms, JSON_FORCE_OBJECT);

	return $json;
}

function gspb_get_taxonomies(WP_REST_Request $request)
{
	$post_type = sanitize_text_field($request->get_param('post_type'));

	$result = \Greenshift\Blocks\ProductTaxonomy::get_taxonomies($post_type);

	return json_encode($result);
}

function gspb_get_all_taxonomies()
{
	$exclude_list = array_flip([
		'nav_menu', 'link_category', 'post_format',
		'elementor_library_type', 'elementor_library_category', 'action-group'
	]);
	$response_data = [];
	$args = [];
	foreach (get_taxonomies($args, 'objects') as $taxonomy => $object) {
		if (isset($exclude_list[$taxonomy])) {
			continue;
		}

		$taxonomy = esc_html($taxonomy);
		$response_data[] = [
			'value'    => $taxonomy,
			'label'  => esc_html($object->label),
		];
	}
	return json_encode($response_data);
}

function gspb_get_post_types()
{
	$post_types = get_post_types(['public' => true], 'objects');
	$result = [];

	foreach ($post_types as $post_type) {
		if (empty(get_object_taxonomies($post_type->name, 'objects'))) continue;
		$result[] = ['value' => $post_type->name, 'label' => $post_type->label];
	}

	return json_encode($result);
}

function gspb_get_post_metas(WP_REST_Request $request)
{
	$post_type = sanitize_text_field($request->get_param('post_type'));
	if (empty($post_type)) $post_type = 'post';
	$exclude_empty = false;
	$exclude_hidden = false;
	global $wpdb;
	$query = "
			SELECT DISTINCT($wpdb->postmeta.meta_key) 
			FROM $wpdb->posts 
			LEFT JOIN $wpdb->postmeta 
			ON $wpdb->posts.ID = $wpdb->postmeta.post_id 
			WHERE $wpdb->posts.post_type = '%s'
		";
	if ($exclude_empty)
		$query .= " AND $wpdb->postmeta.meta_key != ''";
	if ($exclude_hidden)
		$query .= " AND $wpdb->postmeta.meta_key NOT RegExp '(^[_0-9].+$)' ";

	$meta_keys = $wpdb->get_col($wpdb->prepare($query, $post_type));

	$result = [];
	foreach ($meta_keys as $meta_key) {
		$result[] = ['value' => $meta_key, 'label' => $meta_key];
	}

	return json_encode($result);
}

function gspb_query_metagetapi(WP_REST_Request $request)
{
	$field = sanitize_text_field($request->get_param('field'));
	$subfield = sanitize_text_field($request->get_param('subfield'));
	$subsubfield = sanitize_text_field($request->get_param('subsubfield'));
	$postId = (int)$request->get_param('postId');
	$type = sanitize_text_field($request->get_param('type'));
	$post_type = sanitize_text_field($request->get_param('post_type'));
	$show_empty = sanitize_text_field($request->get_param('show_empty'));
	$prefix = sanitize_text_field($request->get_param('prefix'));
	$postfix = sanitize_text_field($request->get_param('postfix'));
	$showtoggle = sanitize_text_field($request->get_param('showtoggle'));
	$repeaternumber = sanitize_text_field($request->get_param('repeaternumber'));
	$acfrepeattype = sanitize_text_field($request->get_param('acfrepeattype'));
	$postprocessor = sanitize_text_field($request->get_param('postprocessor'));
	$repeaterArray = $request->get_param('repeaterArray');
	$icon = $request->get_param('icon');

	if ($post_type && $postId == 0) {
		$latest_cpt = get_posts("post_type='.$post_type.'&numberposts=1");
		$postId = $latest_cpt[0]->ID;
	}

	$value = gspb_query_get_custom_value(array('field' => $field, 'subfield' => $subfield, 'subsubfield' => $subsubfield, 'post_id' => $postId, 'type' => $type, 'show_empty' => $show_empty, 'prefix' => $prefix, 'postfix' => $postfix, 'showtoggle' => $showtoggle, 'post_type' => $post_type, 'repeaternumber' => $repeaternumber, 'acfrepeattype' => $acfrepeattype, 'icon' => $icon, 'postprocessor' => $postprocessor, 'repeaterArray' => $repeaterArray));

	return json_encode($value);
}

function gspb_query_thumbelementapi(WP_REST_Request $request)
{
	$type = sanitize_text_field($request->get_param('type'));
	$postfix = sanitize_text_field($request->get_param('postfix'));
	$postId = (int)$request->get_param('postId');
	$maxtemp = intval($request->get_param('maxtemp'));
	$tempscale = sanitize_text_field($request->get_param('tempscale'));

	$value = gspb_query_thumb_counter(array('type' => $type, 'postfix' => $postfix, 'post_id' => $postId, 'maxtemp' => $maxtemp, 'tempscale' => $tempscale));
	return json_encode($value);
}

function gspb_query_wishlistelementapi(WP_REST_Request $request)
{
	$type = sanitize_text_field($request->get_param('type'));
	$icontype = sanitize_text_field($request->get_param('icontype'));
	$postId = (int)$request->get_param('postId');
	$wishlistadd = sanitize_text_field($request->get_param('wishlistadd'));
	$wishlistadded = sanitize_text_field($request->get_param('wishlistadded'));
	$wishlistpage = sanitize_text_field($request->get_param('wishlistpage'));
	$loginpage = sanitize_text_field($request->get_param('loginpage'));
	$noitemstext = sanitize_text_field($request->get_param('noitemstext'));

	$value = gspb_query_wishlist(array(
		'type' => $type, 'icontype' => $icontype, 'post_id' => $postId,
		'wishlistadd' => $wishlistadd, 'wishlistadded' => $wishlistadded, 'wishlistpage' => $wishlistpage, 'loginpage' => $loginpage, 'noitemstext' => $noitemstext
	));
	return json_encode($value);
}

if (!function_exists('gspb_get_user_ip')) {
	function gspb_get_user_ip()
	{
		foreach (array('HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_X_CLUSTER_CLIENT_IP', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR') as $key) {
			if (array_key_exists($key, $_SERVER) === true) {
				$ip = $_SERVER[$key];
				if (strpos($ip, ',') !== false) {
					$ip = explode(',', $ip);
					$ip = $ip[0];
				}
				if ($ip) {
					substr_replace($ip, 0, -1);
				} //GDRP        		
				return esc_attr($ip);
			}
		}
		return '127.0.0.3';
	}
}

function GSPB_make_dynamic_image($dynamic_style, $attrs, $block, $attribute, $attributeimage)
{
	$imageoriginal = $attributeimage;
	$image_url = $postid = '';

	if (!empty($attribute['dynamicSource']) && $attribute['dynamicSource'] == 'definite_item') {
		if ($attribute['dynamicType'] == 'taxonomy') {
			$postid = !empty($attribute['dynamicTaxonomyId']) ? $attribute['dynamicTaxonomyId'] : '';
		} else {
			if (!empty($attribute['dynamicPostId'])) {
				$postid = (int)$attribute['dynamicPostId'];
			}
		}
	} else {
		if (!empty($attribute['dynamicType']) && $attribute['dynamicType'] == 'taxonomy') {
			if (is_tax() || is_category() || is_tag()) {
				$postid = get_queried_object()->term_id;
			} else {
				global $post;
				if (is_object($post)) {
					$postid = $post->ID;
					$fieldslug = esc_attr($attribute['dynamicTaxonomy']);
					if ($fieldslug) {
						$term_ids =  wp_get_post_terms($postid, $fieldslug, array("fields" => "ids"));
						if (!empty($term_ids) && !is_wp_error($term_ids)) {
							$term_id = $term_ids[0];
							$postid = $term_id;
						}
					}
				}
			}
		} else {
			global $post;
			if (is_object($post)) {
				$postid = $post->ID;
			}
		}
	}
	if (!$postid) return $dynamic_style;
	if ($attribute['dynamicType'] === 'custom') {
		if (!empty($attribute['dynamicField'])) {
			$field = esc_attr($attribute['dynamicField']);
			$fieldvalue = GSPB_get_custom_field_value($postid, $field, 'no');
			if (is_numeric($fieldvalue)) $fieldvalue = wp_get_attachment_url($fieldvalue);
			$image_url = esc_url($fieldvalue);
		}
	} else if ($attribute['dynamicType'] == 'featured') {
		$image_url = get_the_post_thumbnail_url($postid, 'full');
	} else if ($attribute['dynamicType'] == 'taxonomy') {
		if (!empty($attribute['dynamicTaxonomyField'])) {
			$field = esc_attr($attribute['dynamicTaxonomyField']);
			$fieldvalue = get_term_meta($postid, $field, true);
			if (is_array($fieldvalue)) $fieldvalue = $fieldvalue[0];
			if (is_numeric($fieldvalue)) $fieldvalue = wp_get_attachment_url($fieldvalue);
			$image_url = esc_url($fieldvalue);
		}
	}
	if ($image_url && $imageoriginal) {
		return str_replace($imageoriginal, $image_url, $dynamic_style);
	} else {
		return '';
	}
	return $dynamic_style;
}

function GSPB_make_dynamic_text($dynamic_text, $attrs, $block, $attribute, $attributetext)
{

	$textoriginal = $attributetext;
	$text_replace = '';

	if (!empty($attribute['dynamicSource']) && $attribute['dynamicSource'] == 'definite_item') {
		if ($attribute['dynamicType'] == 'taxonomy') {
			$postid = $attribute['dynamicTaxonomyId'];
		} else {
			if (!empty($attribute['dynamicPostId'])) {
				$postid = (int)$attribute['dynamicPostId'];
			}
		}
	} else {
		if (!empty($attribute['dynamicType']) && $attribute['dynamicType'] == 'taxonomy') {
			if (is_tax() || is_category() || is_tag()) {
				$postid = get_queried_object()->term_id;
			}
		} else {
			global $post;
			if (is_object($post)) {
				$postid = $post->ID;
			}
		}
	}
	if ($attribute['dynamicType'] === 'custom') {
		if (!$postid) return $dynamic_text;
		$fieldvalue = '';
		if (!empty($attribute['dynamicField'])) {
			$field = esc_attr($attribute['dynamicField']);
			$fieldvalue = GSPB_get_custom_field_value($postid, $field);
			$text_replace = $fieldvalue;
		}
	} else if ($attribute['dynamicType'] == 'sitedata' && !empty($attribute['dynamicSiteData'])) {
		$field = esc_attr($attribute['dynamicSiteData']);

		if ($field == 'name' || $field == 'description') {
			$text_replace = get_bloginfo($field);
		} else if ($field == 'year') {
			$text_replace = date_i18n("Y");
		} else if ($field == 'month') {
			$text_replace = date_i18n("F");
		}
	} else if ($attribute['dynamicType'] == 'authordata' && !empty($attribute['dynamicAuthorData'])) {
		if (!$postid) return $dynamic_text;
		$field = esc_attr($attribute['dynamicAuthorData']);
		$resultObj = get_post($postid);
		if (is_object($resultObj)) {
			$authorID = $resultObj->post_author;
			if ($field == 'meta' && !empty($attribute['dynamicAuthorField'])) {
				$text_replace = get_user_meta($authorID, $attribute['dynamicAuthorField'], true);
			} else {
				$text_replace = get_the_author_meta($field, $authorID);
			}
		}
	} else if ($attribute['dynamicType'] == 'taxonomyvalue' && !empty($attribute['dynamicTaxonomyValue'])) {
		if (!$postid) return $dynamic_text;
		$field = esc_attr($attribute['dynamicTaxonomyValue']);
		$divider = !empty($attribute['dynamicTaxonomyDivider']) ? '<span class="gspb_tax_spacer">' . $attribute['dynamicTaxonomyDivider'] . '</span>' : ', ';
		$linkrender = !empty($attribute['dynamicTaxonomyLink']) ? $attribute['dynamicTaxonomyLink'] : false;
		if ($linkrender) {
			$term_list = get_the_term_list($postid, $field, '', $divider, '');
			if (!is_wp_error($term_list)) {
				$text_replace = $term_list;
			}
		} else {
			$terms = get_the_terms($postid, $field);
			if ($terms && !is_wp_error($terms)) {
				$term_slugs_arr = array();
				foreach ($terms as $term) {
					$term_slugs_arr[] = '' . $term->name . '';
				}
				$terms_slug_str = join($divider, $term_slugs_arr);
				$text_replace = $terms_slug_str;
			}
		}
	} else if ($attribute['dynamicType'] == 'postdata' && !empty($attribute['dynamicPostData'])) {
		if (!$postid) return $dynamic_text;
		$field = esc_attr($attribute['dynamicPostData']);

		$resultObj = get_post($postid);
		if (is_object($resultObj)) $text_replace = $resultObj->$field;
		if ($field == 'post_date') {
			$text_replace = get_the_date('', $postid);
		} else if ($field == 'post_modified') {
			$text_replace = get_the_modified_date('', $postid);
		}
	} else if ($attribute['dynamicType'] == 'taxonomy' && !empty($attribute['dynamicTaxonomyField'])) {
		if (!$postid) return $dynamic_text;
		$field = esc_attr($attribute['dynamicTaxonomyField']);

		if ($field == 'name') {
			$taxobj = get_term($postid);
			if (!is_wp_error($taxobj)) {
				$fieldvalue = $taxobj->name;
			}
		} else if ($field == 'description') {
			$taxobj = get_term($postid);
			if (!is_wp_error($taxobj)) {
				$fieldvalue = $taxobj->description;
			}
		} else {
			$fieldvalue = get_term_meta($postid, $field, true);
		}
		if (is_array($fieldvalue)) $fieldvalue = $fieldvalue[0];
		$text_replace = $fieldvalue;
	}
	if ($text_replace && $textoriginal) {
		return preg_replace('/<dynamictext>[\s\S]*?<\/dynamictext>/', '<dynamictext>' . $text_replace . '</dynamictext>', $dynamic_text);
	}
	return $dynamic_text;
}

function GSPB_make_dynamic_video($html, $attrs, $block, $attribute, $attributevideo)
{
	$videooriginal = $attributevideo;
	$video_url = '';

	global $post;
	if (is_object($post)) {
		$postid = $post->ID;
	}
	if (!empty($attribute) && $postid) {
		$field = esc_attr($attribute);
		$fieldvalue = GSPB_get_custom_field_value($postid, $field, 'no');
		if (is_numeric($fieldvalue)) $fieldvalue = wp_get_attachment_url($fieldvalue);
		$video_url = esc_url($fieldvalue);
	}
	if ($video_url && $videooriginal) {
		return str_replace($videooriginal, $video_url, $html);
	}
	return $html;
}
function GSPB_make_dynamic_link($html, $attrs, $block, $attribute, $attributelink)
{
	$linkoriginal = $attributelink;
	$link_url = '';

	global $post;
	if (is_object($post)) {
		$postid = $post->ID;
	}
	if (!empty($attrs['dynamicType']) && $attrs['dynamicType'] == 'permalink') {
		$link_url = get_the_permalink($postid);
	} else {
		if (!empty($attribute) && $postid) {
			$field = esc_attr($attribute);
			$fieldvalue = GSPB_get_custom_field_value($postid, $field, 'no');
			if (is_numeric($fieldvalue)) $fieldvalue = wp_get_attachment_url($fieldvalue);
			$link_url = esc_url($fieldvalue);
			$link_url = apply_filters('greenshiftseo_url_filter', $link_url);
		}
	}
	if ($link_url) {
		return preg_replace('/href\s*=\s*"([^"]*)"/i', 'href="' . $link_url . '"', $html);
	} else {
		return '';
	}
	return $html;
}

function GSPB_make_dynamic_flatvalue($html, $attrs, $block, $attribute, $attributereplace)
{
	$textoriginal = $attributereplace;
	$value = '';

	global $post;
	if (is_object($post)) {
		$postid = $post->ID;
	}
	if (!empty($attrs['dynamicType']) && $attrs['dynamicType'] == 'permatext') {
	} else {
		if (!empty($attribute) && $postid) {
			$field = esc_attr($attribute);
			$value = GSPB_get_custom_field_value($postid, $field);
		}
	}
	if ($value && $textoriginal) {
		return str_replace($textoriginal, $value, $html);
	}
	return $html;
}

function GSPB_get_custom_field_value($postid, $field, $divider = ', ', $type = 'custom')
{
	if (strpos($field, '[') !== false) {
		$fieldarray = explode('[', $field);
		$cleanfield = $fieldarray[0];
		$pattern = '/\[(\'[^\']*\'|"[^"]*"|[^\]]*)\]/';
		preg_match_all($pattern, $field, $matches);
		$matches = $matches[1];
		if (!empty($matches) && is_array($matches)) {
			if ($type == 'option') {
				$fieldvalue = get_option($cleanfield);
			} else {
				$fieldvalue = get_post_meta($postid, $cleanfield, true);
			}
			foreach ($matches as $match) {
				$match = str_replace("'", '', $match);
				$match = str_replace('"', '', $match);
				if (isset($fieldvalue[$match])) {
					$fieldvalue = $fieldvalue[$match];
				}
			}
		}
	} else {
		if ($type == 'option') {
			$fieldvalue = get_option($field);
		} else {
			$fieldvalue = get_post_meta($postid, $field, true);
		}
	}

	$fieldvalue = GSPB_field_array_to_value($fieldvalue, $divider);
	return apply_filters('greenshift_dynamic_field_output', $fieldvalue);
}

function GSPB_field_array_to_value($fieldvalue, $divider)
{
	if (is_array($fieldvalue) && count($fieldvalue) == count($fieldvalue, COUNT_RECURSIVE)) {
		if ($divider == 'list') {
			$fieldvalue = '<ul><li>' . implode('</li><li>', $fieldvalue) . '</li></ul>';
		} else if ($divider == 'no') {
			$fieldvalue = $fieldvalue[0];
		} else if ($divider == 'flat') {
			//nothing to do
		} else {
			$fieldvalue = implode($divider, $fieldvalue);
		}
	}
	return $fieldvalue;
}
