<?php


namespace greenshiftwoo\Blocks;

defined('ABSPATH') or exit;


class ProductGallery
{

	public function __construct()
	{
		add_action('init', array($this, 'init_handler'));
	}

	public function init_handler()
	{
		register_block_type(
			__DIR__,
			array(
				'render_callback' => array($this, 'render_block'),
				'attributes'      => $this->attributes
			)
		);
	}

	public $attributes = array(
		'id' => array(
			'type'    => 'string',
			'default' => null,
		),
		'inlineCssStyles' => array(
			'type'    => 'string',
			'default' => '',
		),
		'animation' => array(
			'type' => 'object',
			'default' => array(),
		),
		'sourceType'       => array(
			'type'    => 'string',
			'default' => 'latest_item',
		),
		'postId'       => array(
			'type'    => 'number',
			'default' => 0,
		),
		'direction'       => array(
			'type'    => 'string',
			'default' => 'horizontal',
		),
		'slidesPerView' => array(
			'type'    => 'array',
			'default' => [4, 3, 3, 3],
		),
		'spaceBetween' => array(
			'type'    => 'array',
			'default' => [10, 10, 10, 10],
		),
	);

	public function render_block($settings = array(), $inner_content = '')
	{
		extract($settings);

		if($sourceType == 'latest_item'){
			global $post;
			if(is_object($post)){
				$postId = $post->ID;
			}else{
				$postId = 0;
			}
		}else{
			$postId = (isset($postId) && $postId > 0) ? (int)$postId : 0;
		}

		$_product = gspbwoo_get_product_object_by_id($postId);

		if (!$_product) return __('Product not found.', 'greenshiftwoo');

		$htmlGallery = $this->gallery_html($_product->get_id(), $slidesPerView, $spaceBetween);

		$htmlGallery = !empty($htmlGallery) ? $htmlGallery : __('Gallery no found.', 'greenshiftwoo');

		$blockId = 'gspb_id-' . $id;
		$blockClassName = 'gspb-product-image-gallery ' . $blockId . ' ' . (!empty($className) ? $className : '') . ' ';

		$out = '<div  class="' . $blockClassName . '"' . gspb_AnimationRenderProps($animation) . ' data-direction="' . $direction . '">';
		$out .= $htmlGallery;
		$out .= '</div>';
		return $out;
	}

	public function gallery_html($product_id, $slidesPerView, $spaceBetween)
	{
		$_product = wc_get_product($product_id);
		$featured_image = $_product->get_image_id();
		$image_ids = $_product->get_gallery_image_ids();
		ob_start();
?>
		<div class="gspb-product-image-gallery-wrap">
			<div style="--swiper-navigation-color: #fff; --swiper-pagination-color: #fff" class="swiper gspb-gallery-full">
				<div class="swiper-wrapper">
					<?php if (!empty($featured_image)) : ?>

						<div class="swiper-slide swiper-slide-main-image">
							<?php echo wp_get_attachment_image($featured_image, 'woocommerce_single', false, [
								'data-main-featured-image-src' => wp_get_attachment_image_src($featured_image, 'woocommerce_single')[0],
								'data-main-featured-image-srcset' => wp_get_attachment_image_srcset($featured_image, 'woocommerce_single'),
							]) ?>
						</div>

						<?php foreach ($image_ids as $image_id) : ?>
							<div class="swiper-slide">
								<?php echo wp_get_attachment_image($image_id, 'woocommerce_single') ?>
							</div>
						<?php endforeach; ?>
					<?php else : ?>
						<div class="swiper-slide">
							<?php echo sprintf('<img src="%s" alt="%s" class="wp-post-image" />', esc_url(wc_placeholder_img_src('woocommerce_single')), esc_html__('Awaiting product image', 'woocommerce')); ?>
						</div>
					<?php endif; ?>
				</div>
			</div>
			<div class="swiper gspb-gallery-thumb" 
				data-slidesperview = "<?php echo $slidesPerView[0];?>"
				data-slidesperviewmd = "<?php echo $slidesPerView[1];?>"
				data-slidesperviewsm = "<?php echo $slidesPerView[2];?>"
				data-slidesperviewxs = "<?php echo $slidesPerView[3];?>"
				data-spacebetween = "<?php echo $spaceBetween[0];?>"
				data-spacebetweenmd = "<?php echo $spaceBetween[1];?>"
				data-spacebetweensm = "<?php echo $spaceBetween[2];?>"
				data-spacebetweenxs = "<?php echo $spaceBetween[3];?>"
			>
				<div class="swiper-wrapper">
					<?php if (!empty($featured_image)) : ?>
						<div class="swiper-slide swiper-slide-main-image">
							<?php echo wp_get_attachment_image($featured_image, 'woocommerce_gallery_thumbnail', false, [
								'data-main-featured-image-src' => wp_get_attachment_image_src($featured_image, 'woocommerce_gallery_thumbnail')[0],
								'data-main-featured-image-srcset' => wp_get_attachment_image_srcset($featured_image, 'woocommerce_gallery_thumbnail'),
							]) ?>
						</div>

						<?php foreach ($image_ids as $image_id) : ?>
							<div class="swiper-slide">
								<?php echo wp_get_attachment_image($image_id, 'woocommerce_gallery_thumbnail') ?>
							</div>
						<?php endforeach; ?>
					<?php endif; ?>
				</div>
			</div>
		</div>
<?php
		$res = ob_get_contents();
		ob_get_clean();
		return $res;
	}
}

new ProductGallery;
