<?php


namespace greenshiftwoo\Blocks;
defined('ABSPATH') OR exit;


class ProductQuickView{

	public function __construct(){
		add_action('init', array( $this, 'init_handler' ));
	}

	public function init_handler(){
		register_block_type(__DIR__, array(
                'render_callback' => array( $this, 'render_block' ),
                'attributes'      => $this->attributes
            )
		);
	}

	public $attributes = array(
		'id' => array(
			'type'    => 'string',
			'default' => null,
		),
		'inlineCssStyles' => array(
			'type'    => 'string',
			'default' => '',
		),
		'animation' => array(
			'type' => 'object',
			'default' => array(),
		),
        'sourceType'       => array(
            'type'    => 'string',
            'default' => 'latest_item',
        ),
        'label'       => array(
            'type'    => 'string',
            'default' => '',
        ),
        'postfix'       => array(
            'type'    => 'string',
            'default' => '',
        ),
        'postId'       => array(
            'type'    => 'number',
            'default' => 0,
        ),
	);

	public function render_block($settings = array(), $inner_content=''){
		extract($settings);

        global $product;

        if(!empty($product)) {
            $product_content = $this->get_product_quick_view_content();
        } else {

            if(!empty($postId)) {
                $args = array(
                    'post_type' => 'product',
                    'posts_per_page' => 1,
                    'post__in' => [$postId]
                );
            } else {
                $args = array(
                    'post_type' => 'product',
                    'posts_per_page' => 1,
                    'orderby' =>'date',
                    'order' => 'DESC'
                );
            }

            $loop = new \WP_Query( $args );
            if ( $loop->have_posts() ) {
                while ( $loop->have_posts() ) : $loop->the_post();
                    $product_content = $this->get_product_quick_view_content();
                endwhile;
            } else {
                return __('Product not found.', 'greenshiftwoo');
            }
            wp_reset_postdata();
        }

		$blockId = 'gspb-product-quick-view-box-id-'.$id;
		$blockClassName = 'gspb-product-quick-view-box-wrapper '.$blockId.' '.(!empty($className) ? $className : '').'';
		$out = '<div class="'.$blockClassName.'"'.gspb_AnimationRenderProps($animation).'>';
			$out .= '<div class="gspb-product-quick-view-popup">';
                $out .= '<div class="gspb-product-quick-view-popup-inner">';
                $out .= '<span class="gspb-product-quick-view-close">×</span>';
                $out .= $product_content;
                $out .= '</div>';
			$out .= '</div>';
            $out .= $inner_content;
		$out .='</div>';

		return $out;
	}

    public function get_product_quick_view_content() {

        global $product;

        if($product !== null) {
            wp_enqueue_script( 'wc-single-product' );
            wp_enqueue_script( 'wc-add-to-cart-variation' );


            if ( current_theme_supports( 'wc-product-gallery-slider' ) ) {
                wp_enqueue_script( 'flexslider' );
            }

            if ( current_theme_supports( 'wc-product-gallery-lightbox' ) ) {
                wp_enqueue_script( 'photoswipe-ui-default' );
                wp_enqueue_style( 'photoswipe-default-skin' );
                add_action( 'wp_footer', 'woocommerce_photoswipe' );
            }

            if ( current_theme_supports( 'wc-product-gallery-zoom' ) ) {
                wp_enqueue_script( 'zoom' );
            }
            ob_start();
            /**
             * Hook: woocommerce_before_single_product.
             *
             * @hooked woocommerce_output_all_notices - 10
             */
            do_action( 'woocommerce_before_single_product' );

            if ( post_password_required() ) {
                echo get_the_password_form(); // WPCS: XSS ok.
                return;
            }
            remove_action('woocommerce_single_product_summary', 'woocommerce_template_single_rating', 10);
            add_action('woocommerce_single_product_summary', [$this, 'quick_view_template_rating'], 9);
            ?>
            <div class="woocommerce">
                <div id="product-<?php the_ID(); ?>" <?php wc_product_class( '', $product ); ?>>

                    <?php
                    /**
                     * Hook: woocommerce_before_single_product_summary.
                     *
                     * @hooked woocommerce_show_product_sale_flash - 10
                     * @hooked woocommerce_show_product_images - 20
                     */
                    do_action( 'woocommerce_before_single_product_summary' );
                    ?>

                    <div class="summary entry-summary">
                        <?php
                        /**
                         * Hook: woocommerce_single_product_summary.
                         *
                         * @hooked woocommerce_template_single_title - 5
                         * @hooked woocommerce_template_single_rating - 10
                         * @hooked woocommerce_template_single_price - 10
                         * @hooked woocommerce_template_single_excerpt - 20
                         * @hooked woocommerce_template_single_add_to_cart - 30
                         * @hooked woocommerce_template_single_meta - 40
                         * @hooked woocommerce_template_single_sharing - 50
                         * @hooked WC_Structured_Data::generate_product_data() - 60
                         */
                        do_action( 'woocommerce_single_product_summary' );
                        ?>

                        <a href="<?php echo get_permalink($product->get_id())?>" class="gspb-link-full-page"><?php echo __('Open full product page', 'greenshiftwoo')?></a>
                    </div>
                </div>
            </div>

            <?php
            $html = ob_get_contents();
            ob_get_clean();
        } else $html = __('Product not found.', 'greenshiftwoo');

        return $html;
    }

    public function quick_view_template_rating() {
        if ( post_type_supports( 'product', 'comments' ) ) {
            include GREENSHIFTWOO_DIR_PATH .'inc/templates/quick-view/rating.php';
        }
    }
}

new ProductQuickView;