document.addEventListener('DOMContentLoaded', function () {

	const galleries = document.querySelectorAll('.gspb-product-image-gallery');

	for (let i = 0; i < galleries.length; i++) {
		const direction = galleries[i].getAttribute('data-direction');
		const swipergallery = galleries[i].querySelector('.gspb-gallery-thumb');
		let slidesPerView = parseFloat(swipergallery.dataset.slidesperview);
		let slidesPerViewMD = parseFloat(swipergallery.dataset.slidesperviewmd);
		let slidesPerViewSM = parseFloat(swipergallery.dataset.slidesperviewsm);
		let slidesPerViewXS = parseFloat(swipergallery.dataset.slidesperviewxs);
		let spaceBetween = parseInt(swipergallery.dataset.spacebetween);
		let spaceBetweenMD = parseInt(swipergallery.dataset.spacebetweenmd);
		let spaceBetweenSM = parseInt(swipergallery.dataset.spacebetweensm);
		let spaceBetweenXS = parseInt(swipergallery.dataset.spacebetweenxs);

		const swiperThumb = new Swiper(swipergallery, {
			loop: false,
            spaceBetween : spaceBetween,
            slidesPerView : slidesPerView,
			freeMode: false,
			watchSlidesProgress: true,
			direction: direction,
			grabCursor: true,
			breakpoints: {
                236: {
                    slidesPerView: slidesPerViewXS ? slidesPerViewXS : slidesPerView,
                    spaceBetween: spaceBetweenXS ? spaceBetweenXS : spaceBetween
                },
                576: {
                    slidesPerView: slidesPerViewSM ? slidesPerViewSM : slidesPerView,
                    spaceBetween: spaceBetweenSM ? spaceBetweenSM : spaceBetween
                },
                768: {
                    slidesPerView: slidesPerViewMD ? slidesPerViewMD : slidesPerView,
                    spaceBetween: spaceBetweenMD ? spaceBetweenMD : spaceBetween
                },
                992: {
                    spaceBetween,
                    slidesPerView
                }
            },
		});
		const swiperFull = new Swiper(galleries[i].querySelector('.gspb-gallery-full'), {
			loop: true,
			spaceBetween: 0,
			grabCursor: true,
			thumbs: {
				swiper: swiperThumb,
			},
		});
	}

	const variationsObj = document.querySelector('.variations_form.cart') !== null ? JSON.parse(document.querySelector('.variations_form.cart').getAttribute('data-product_variations')) : '';

	if (variationsObj.length) {
		document.querySelector('.variations_form.cart').addEventListener('change', () => changeImageByVariation('change'))
		document.querySelector('.reset_variations').addEventListener('click', () => changeImageByVariation('click'))

		function changeImageByVariation(event) {
			if (event === 'click') {
				setTimeout(mainFunc, 100);
			} else mainFunc();

			function mainFunc() {
				const currentVariationId = document.querySelector('input.variation_id').getAttribute('value');
				const currentVariation = variationsObj.filter((e) => {
					return parseInt(e.variation_id) === parseInt(currentVariationId);
				})

				if (galleries.length) {
					if (currentVariation.length) {

						const single_src = currentVariation[0].image.src;
						const thumb_src = currentVariation[0].image.gallery_thumbnail_src;
						const srcset = currentVariation[0].image.srcset;

						for (let i = 0; i < galleries.length; i++) {
							const mainImg = galleries[i].querySelector('.gspb-gallery-full .swiper-slide-main-image img');
							const thumbImg = galleries[i].querySelector('.gspb-gallery-thumb .swiper-slide-main-image img');

							mainImg.setAttribute('src', single_src)
							mainImg.setAttribute('srcset', srcset)
							thumbImg.setAttribute('src', thumb_src)
							thumbImg.setAttribute('srcset', srcset)
						}

					} else {
						for (let i = 0; i < galleries.length; i++) {
							const mainImg = galleries[i].querySelector('.gspb-gallery-full .swiper-slide-main-image img');
							const thumbImg = galleries[i].querySelector('.gspb-gallery-thumb .swiper-slide-main-image img');

							mainImg.setAttribute('src', mainImg.getAttribute('data-main-featured-image-src'))
							mainImg.setAttribute('srcset', mainImg.getAttribute('data-main-featured-image-srcset'))
							thumbImg.setAttribute('src', thumbImg.getAttribute('data-main-featured-image-src'))
							thumbImg.setAttribute('srcset', thumbImg.getAttribute('data-main-featured-image-srcset'))
						}
					}
				}
			}
		}
	}
}, false)

